@@ -8,6 +8,7 @@ import os
 import re
 import pandas as pd
 import time
+import connectorx as cx
 
 
 def find_tables_by_column_name(column_name: str, database: Optional[str] = None, verbose: bool = False) -> pd.DataFrame:
@@ -32,6 +33,7 @@ def find_tables_by_column_name(column_name: str, database: Optional[str] = None,
 
     return df_by_column
 
+
 def send_metadata_to_redshift(table_name: str, database: Optional[str] = None, file_path: str = "table_metadata.json"):
     """
     Use this function to send the Table and Column Descriptions to Redshift. There is a standard JSON file which this
@@ -176,11 +178,11 @@ def send_to_redshift(database: str,
         if database == "staging":
             so.bucket = kwargs.get("bucket", os.environ.get("S3_BUCKET_DEV"))
             if so.bucket is None:
-                print("❌ Environment variable missing: S3_BUCKET_DEV")
+                raise ValueError("❌ Environment variable missing: S3_BUCKET_DEV")
         else:
             so.bucket = kwargs.get("bucket", os.environ.get("S3_BUCKET_DATAMART"))
             if so.bucket is None:
-                print("❌ Environment variable missing: S3_BUCKET_DATAMART")
+                raise ValueError("❌ Environment variable missing: S3_BUCKET_DATAMART")
     else:
         so.bucket = from_bucket
 
@@ -200,15 +202,13 @@ def send_to_redshift(database: str,
             send_metadata_to_redshift(table_name=table, database=database, file_path=json_path)
         except ValueError:
             print(f"🔍 Please check if the name is correct or set send_metadata to false.")
-            ro.conn.close()
             raise
         except FileNotFoundError:
             print(f"⁉️ The JSON file containing all metadata was not found. Please, either check if the "
                   f"file path passed is correct, or set send_metadata to False.")
-            ro.conn.close()
             raise
-
-    ro.conn.close()
+        finally:
+            ro.conn.close()
 
 
 def read_from_redshift(database: str, method: str, **kwargs) -> pd.DataFrame:
@@ -257,15 +257,17 @@ class RedshiftOperator:
         >>> ro.read_from_redshift(schema="my_schema", table="my_table", limit=10)
     """
 
-    def __init__(self, database: str = os.environ.get("REDSHIFT_DB"), debug: bool = False) -> None:
+    def __init__(self,
+                 database: str = os.environ.get("REDSHIFT_DB"), debug: bool = False) -> None:
         self.redshift_user = os.environ.get("REDSHIFT_USER")
         self.redshift_password = os.environ.get("REDSHIFT_PASSWORD")
         self.redshift_host = os.environ.get("REDSHIFT_HOST")
+        self.redshift_port = os.environ.get("REDSHIFT_PORT")
         self.redshift_database = database
         if database is None:
             raise ValueError(f"❌Input database not defined in .env")
         self.engine = create_engine(
-            f"redshift+psycopg2://{self.redshift_user}:{self.redshift_password}@{self.redshift_host}:5439/{self.redshift_database}",
+            f"redshift+psycopg2://{self.redshift_user}:{self.redshift_password}@{self.redshift_host}:{self.redshift_port}/{self.redshift_database}",
             echo=debug,
         )
         self.conn = self.engine.connect()
@@ -313,10 +315,15 @@ class RedshiftOperator:
         read method
     """
 
-    def read_from_redshift(self, limit: bool = None, select: str = "*") -> pd.DataFrame:
+    def read_from_redshift(self, limit: bool = None, select: str = "*", method: str = "pandas") -> pd.DataFrame:
         sql_limit = limit if limit else "NULL"
         query = f"SELECT {select} FROM {self._schema}.{self._table} LIMIT {sql_limit}"
-        df_result = pd.read_sql_query(query, self.engine)
+        if method == "pandas":
+            df_result = pd.read_sql_query(query, self.engine)
+        if method == "ctx":
+            df_result = cx.read_sql(
+                f"redshift+psycopg2://{self.redshift_user}:{self.redshift_password}@{self.redshift_host}:{self.redshift_port}/{self.redshift_database}",
+                query, partition_num=8, return_type="pandas", )
         return df_result
 
     def read_sql_from_redshift(self, sql_query: str) -> pd.DataFrame:
@@ -475,50 +482,52 @@ class RedshiftOperator:
         self.conn.execution_options(isolation_level="AUTOCOMMIT").execute(text(query))
 
     def get_DDL(self,
+                schema_names: Optional[List[str]] = None,
+                avoid_schema_names: Optional[List[str]] = None,
+                table_names: Optional[List[str]] = None,
+                avoid_table_names: Optional[List[str]] = None,
                 verbose: bool = True,
-                schema_names: List[str] = [],
-                table_names: List[str] = [],
-                output_to: str = None,
-                not_include: Optional = None):
+                output_to: Optional[str] = None):
         """
         This function makes a Redshift request to retrieve the v_generate_tbl_ddl table responsible for
         stocking the ddl queries from all the redshift tables.
 
-        :param verbose: Print details.
         :param schema_names: List os schemas to be considered if customization is needed.
+        :param avoid_schema_names: List of schemas to be excluded if any.
         :param table_names: List of tables to be considered if customization is needed.
+        :param avoid_table_names:List of tables to be excluded if any.
+        :param verbose: Print details.
         :param output_to: If we want to export this table somewhere, tha path must be passed here.
-        :param not_include: List of schemas to be excluded if any.
         :return:
         """
         # Generate the DDL table.
         self.generate_DDL()
-        time.sleep(2)  # Allow a little time for the query to finish executing.
+        time.sleep(2)  # Allow a little of time for the query to finish executing.
 
-        if not_include is None:
-            not_include = [
-                "public", "pv_reference", "platform", "pv_intermediary_tables", "pg_catalog", "information_schema"
-            ]
+        if avoid_schema_names is None:
+            avoid_schema_names = ["public", "pv_reference", "platform", "pv_intermediary_tables", "pg_catalog",
+                                  "information_schema"]
 
         print("⏳ Generating the DDL Table. It can take a while...")
         query = f"select * from admin.v_generate_tbl_ddl"
-
-        # Since
         result = self.conn.execute(text(query)).all()
-
-        if schema_names == []:
+        if not schema_names:
             schema_names = list(set([record.schemaname for record in result]))
 
         dfs = []
-        for schema in set(schema_names).difference(not_include):
+        for schema in set(schema_names).difference(avoid_schema_names):
             if verbose:
                 print(schema)
-            if table_names == []:
-                table_names_ = list(set([record.tablename for record in result if record.schemaname == schema]))
+            if not table_names:
+                table_names_list = list(set([record.tablename for record in result if record.schemaname == schema]))
+                if avoid_table_names:  # Exclude the tables if any.
+                    table_names_list = set(table_names_list).difference(avoid_table_names)
             else:
-                table_names_ = list(set(table_names) &
+                # Intercession between both sets.
+                table_names_list = list(set(table_names) &
                                     set([record.tablename for record in result if record.schemaname == schema]))
-            for table in table_names_:
+
+            for table in table_names_list:
                 if verbose:
                     print(f" - {table}")
                 corrected_query = ""
@@ -536,7 +545,8 @@ class RedshiftOperator:
                     .replace(".type ", '."type" ') \
                     .replace(".role ", '."role" ') \
                     .replace(".provider ", '."provider" ') \
-                    .replace(".location ", '."location" ')
+                    .replace(".location ", '."location" ') \
+                    .replace(".index ", '."index" ')
 
                 corrected_query = sqlparse.split(corrected_query)
 
@@ -604,6 +614,7 @@ class RedshiftOperator:
 
         print("DDL Table generated!")
         self.ddl_table = pd.concat(dfs, ignore_index=True).reset_index(drop=True)
+        self.ddl_table.sort_values(["schema_name", "table_name"], ascending=[1, 1], ignore_index=True, inplace=True)
 
         if output_to is not None:
             print("Exporting table to folder.")
